local aName, devnull = ...
local _G = _G

-- check to see if required libraries are loaded
assert(LibStub, aName.." requires LibStub")
for _, lib in pairs{"CallbackHandler-1.0", "AceAddon-3.0", "AceConsole-3.0", "AceEvent-3.0", "AceHook-3.0",  "AceLocale-3.0", "LibBabble-Zone-3.0", "LibTourist-3.0", "AceDB-3.0", "AceDBOptions-3.0", "AceGUI-3.0",  "AceConfig-3.0", "AceConfigCmd-3.0", "AceConfigRegistry-3.0", "AceConfigDialog-3.0",} do
	assert(LibStub:GetLibrary(lib, true), aName.." requires "..lib)
end

-- create the addon
LibStub("AceAddon-3.0"):NewAddon(devnull, aName, "AceConsole-3.0", "AceEvent-3.0", "AceHook-3.0")

-- specify where debug messages go
devnull.debugFrame = ChatFrame7
devnull.debugLevel = 0

-- Get Locale
local L = LibStub("AceLocale-3.0"):GetLocale("devnull")
local ZL = LibStub("LibBabble-Zone-3.0"):GetLookupTable()
local T = LibStub("LibTourist-3.0")

local nullCities = {
	[ZL["Stormwind City"]] = true,
	[ZL["Ironforge"]] = true,
	[ZL["Darnassus"]] = true,
	[ZL["Orgrimmar"]] = true,
	[ZL["Undercity"]] = true,
	[ZL["Thunder Bluff"]] = true,
	[ZL["The Exodar"]] = true, -- TBC
	[ZL["Silvermoon City"]] = true, -- TBC
	[ZL["Shattrath City"]] = true, -- TBC
	[ZL["Dalaran"]] = true, -- WotLK
}
local nullTowns = {
	[ZL["Booty Bay"]] = true, 
	[ZL["Everlook"]] = true,
	[ZL["Gadgetzan"]] = true,
	[ZL["Ratchet"]] = true,
	[L["Goldshire"]] = true,
	[L["Area 52"]] = true, -- TBC
	[L["Valgarde"]] = true, -- WotLK
	[L["Fort Wildervar"]] = true, -- WotLK
	[L["Theramore Isle"]] = true,
	[L["Foothold Citadel"]] = true,
}
local icecrownInstances = {
	[ZL["Icecrown Citadel"]] = true,
	[ZL["Halls of Reflection"]] = true, -- Frozen Halls
	[ZL["Pit of Saron"]] = true, -- Frozen Halls
	[ZL["The Forge of Souls"]] = true, -- Frozen Halls
}
local checkEvent = {
    ["ZONE_CHANGED"] = true,
    ["ZONE_CHANGED_NEW_AREA"] = true,
}

function devnull:OnInitialize()
	self:LevelDebug(2, "OnInitialize")

--[===[@debug@
	self:Print("Debugging is enabled")
	self:LevelDebug(1, "Debugging is enabled")
--@end-debug@]===]

	local defaults = { profile = {
		CHAT_MSG_YELL          = true,
		CHAT_MSG_MONSTER_YELL  = true,
		CHAT_MSG_MONSTER_SAY   = true,
		CHAT_MSG_TEXT_EMOTE    = false,
		CHAT_MSG_MONSTER_EMOTE = true,
		CHAT_MSG_TRADESKILLS   = false,
		CHAT_MSG_ACHIEVEMENT   = false,
		CHAT_MSG_PET_INFO      = false,
		noDuel                 = true,
		noDrunk                = true,
		noDiscovery            = true,
		chatback               = true,
		inInstance             = false,
		-- ChatFrame1 channel settings
		cf1Channels = {
		   [L["General"]]          = false,
		   [L["Trade"]]            = false,
		   [L["LocalDefense"]]     = false,
		   [L["WorldDefense"]]     = false,
		   [L["GuildRecruitment"]] = false,
	    },
	   -- Instance channel settings
		iChannels = {
		   [L["General"]]      = true,
		   [L["LocalDefense"]] = true,
		   [L["WorldDefense"]] = true,
	    },
	}}

	self.db = LibStub("AceDB-3.0"):New("devnullDB", defaults, "Default")

	local db = self.db.profile

	local optTables = {

		General = {	
	    	name = aName,
			type = "group",
			args = {
				desc = {
					type = "description",
					order = 1,
					name = L["shhhh"] .." - "..GetAddOnMetadata("devnull", "Version").. "\n",
				},
				longdesc = {
					type = "description",
					order = 2,
					name = L["Mutes various chat related annoyances while in capital cities and instances"] .. "\n",
				},

				chat = {
					type = 'toggle',
					order = 3,
					name = L["Chat Feedback"],
					desc = L["Print chat message when toggling mute."],
					get = function(info) return db.chatback end,
					set = function(info, value) db.chatback = value end,
				},
				Mutes = {
					type = "group",
					order = 4,
					inline = true,
					name = L["Mute Options"],
					desc = L["Toggle Chat type(s) to mute."],
					args = {
						yell = {
							type = 'toggle',
							order = 1,
							name = L["Yells"],
							desc = L["Mute Yells."],
							get = function(info) return db.CHAT_MSG_YELL end,
							set = function(info, value) db.CHAT_MSG_YELL = value db.CHAT_MSG_MONSTER_YELL = value end,
						},
						npc = {
							type = 'toggle',
							order = 2,
							name = L["NPCs"],
							desc = L["Mute NPC chat/yells."],
							get = function(info) return db.CHAT_MSG_MONSTER_SAY end,
							set = function(info, value) db.CHAT_MSG_MONSTER_SAY = value end,
						},
						duel = {
							type = 'toggle',
							order = 3,
							name = L["Duels"],
							desc = L["Mute Duel info."],
							get = function(info) return db.noDuel end,
							set = function(info, value) db.noDuel = value end,
						},
						drunk = {
							type = 'toggle',
							order = 4,
							name = L["Drunks"],
							desc = L["Mute Drunken info."],
							get = function(info) return db.noDrunk end,
							set = function(info, value) db.noDrunk = value end,
						},
						emote = {
							type = 'toggle',
							order = 5,
							name = L["Emotes"],
							desc = L["Mute Emotes."],
							get = function(info) return db.CHAT_MSG_TEXT_EMOTE end,
							set = function(info, value) db.CHAT_MSG_TEXT_EMOTE = value db.CHAT_MSG_MONSTER_EMOTE = value end,
				        },
				        ts = {
							type = 'toggle',
							order = 6,
				    		name = L["Tradeskills"],
				        	desc = L["Mute Tradeskills."],
				    		get = function(info) return db.CHAT_MSG_TRADESKILLS end,
							set = function(info, value) db.CHAT_MSG_TRADESKILLS = value end,
				        },
				        ga = {
							type = 'toggle',
							order = 7,
				    		name = L["Achievement gains"],
				        	desc = L["Mute Achievement gains."],
				    		get = function(info) return db.CHAT_MSG_ACHIEVEMENT end,
							set = function(info, value) db.CHAT_MSG_ACHIEVEMENT = value end,
				        },
				        pet = {
							type = 'toggle',
							order = 7,
				    		name = L["Pet Info"],
				        	desc = L["Mute Pet Info."],
				    		get = function(info) return db.CHAT_MSG_PET_INFO end,
							set = function(info, value) db.CHAT_MSG_PET_INFO = value end,
				        },
				        iGen = {
							type = 'toggle',
							order = 8,
							width = "double",
				    		name = L["General chat in Instances/Battlegrounds"],
				        	desc = L["Mute General chat in Instances/Battlegrounds."],
				    		get = function(info) return db.iChannels[L["General"]] end,
							set = function(info, value) db.iChannels[L["General"]] = value end,
				        },
						discovery = {
							type = 'toggle',
							order = 4,
							name = L["Discoveries"],
							desc = L["Mute Discovery info."],
							get = function(info) return db.noDiscovery end,
							set = function(info, value) db.noDiscovery = value end,
						},
					},
				},
			},
		},
	}

	-- add DB profile options
	optTables.Profiles = LibStub("AceDBOptions-3.0"):GetOptionsTable(self.db)

	-- option tables list
	local optNames = {
		"Profiles"
	}
	
	-- register the options tables and add them to the blizzard frame
	local ACR = LibStub("AceConfigRegistry-3.0")
	local ACD = LibStub("AceConfigDialog-3.0")

	LibStub("AceConfig-3.0"):RegisterOptionsTable(aName, optTables.General, {aName, "dn"})
	self.optionsFrame = ACD:AddToBlizOptions(aName, aName)

	-- register the options, add them to the Blizzard Options
	-- build the table used by the chatCommand function
	local optCheck = {}
	for _, v in ipairs(optNames) do
--		self:Debug("options: [%s]", v)
		local optTitle = (" "):join(aName, v)
		ACR:RegisterOptionsTable(optTitle, optTables[v])
		self.optionsFrame[L[v]] = ACD:AddToBlizOptions(optTitle, L[v], aName)
		optCheck[v:lower()] = v
	end

	-- Slash command handler
	local function chatCommand(input)

		if not input or input:trim() == "" then
			-- Open general panel if there are no parameters
			InterfaceOptionsFrame_OpenToCategory(devnull.optionsFrame)
		elseif optCheck[input:lower()] then
			InterfaceOptionsFrame_OpenToCategory(devnull.optionsFrame[optCheck[input:lower()]])
		else
			LibStub("AceConfigCmd-3.0"):HandleCommand(aName, aName, input)
		end
	end

	-- Register slash command handlers
	self:RegisterChatCommand(aName, chatCommand)
	self:RegisterChatCommand("dn", chatCommand)

end

function devnull:OnEnable()
	self:LevelDebug(2, "OnEnable")

	self:RegisterEvent("ZONE_CHANGED", "CheckZone")
	self:RegisterEvent("ZONE_CHANGED_NEW_AREA", "CheckZone")

	self:CheckZone(sAFI)
	
	-- handle profile changes
	self.db.RegisterCallback(self, "OnProfileChanged", "ReloadAddon")
	self.db.RegisterCallback(self, "OnProfileCopied", "ReloadAddon")
	self.db.RegisterCallback(self, "OnProfileReset", "ReloadAddon")

end

function devnull:OnDisable()
	self:LevelDebug(2, "OnDisable")
	
	-- turn channels back on
    for channel, on in pairs(self.db.profile.cf1Channels) do
        if on then ChatFrame_AddChannel(ChatFrame1, channel) end
    end
	
	self:UnregisterAllEvents()

end

function devnull:ReloadAddon(callback)

--	self:Debug("ReloadAddon:[%s]", callback)

	StaticPopupDialogs["devnull_Reload_UI"] = {
		text = L["Confirm reload of UI to activate profile changes"],
		button1 = OKAY,
		button2 = CANCEL,
		OnAccept = function()
			ReloadUI()
		end,
		OnCancel = function(this, data, reason)
			if reason == "timeout" or reason == "clicked" then
				self:CustomPrint(1, 1, 0, nil, nil, nil, "The profile '"..devnull.db:GetCurrentProfile().."' will be activated next time you Login or Reload the UI")
			end
		end,
		timeout = 0,
		whileDead = 1,
		exclusive = 1,
		hideOnEscape = 1
	}
	StaticPopup_Show("devnull_Reload_UI")

end

function devnull:CheckZone(sAFI)
	self:LevelDebug(1, "CheckZone: [%s]", event or sAFI)
    self:LevelDebug(2,"You Are Here: [%s:%s]", GetRealZoneText() or "<Anon>", GetSubZoneText() or "<Anon>")
    self:LevelDebug(3,"inInstance#1: []%s, %s, %s, %s]", self.db.profile.inInstance, T:IsBattleground(GetRealZoneText()),  T:IsInstance(GetRealZoneText()), icecrownInstances[GetRealZoneText()])
 
    --> Pre Event Handler <--
    -- if entering a new area or just been loaded or come out of standby
    if checkEvent[event] or sAFI then
		if self.db.profile.inInstance then
            self.db.profile.inInstance = false
        else
        -- otherwise save the current channel settings for Chat Frame 1
            for key, _ in pairs(self.db.profile.cf1Channels) do
                self.db.profile.cf1Channels[key] = false
            end
            local cwc = {GetChatWindowChannels(1)}
            for  i = 1, #cwc, 2 do
	           self:LevelDebug(3, "cwc: [%s]", cwc[i])
	           self.db.profile.cf1Channels[cwc[i]] = true
            end
        end
    end

    self:LevelDebug(3,"inInstance#2: []%s, %s, %s, %s]", self.db.profile.inInstance, T:IsBattleground(GetRealZoneText()),  T:IsInstance(GetRealZoneText()), icecrownInstances[GetRealZoneText()])

    --> Event Handler <--    
	if nullCities[GetRealZoneText()] or nullTowns[GetSubZoneText()] then
		if not self:IsHooked("ChatFrame_OnEvent") then
		  self:RawHook("ChatFrame_OnEvent", true)
		  if self.db.profile.chatback then self:Print(L["enabled"]) end
        end
	else
		if self:IsHooked("ChatFrame_OnEvent") then
			self:Unhook("ChatFrame_OnEvent")
			if self.db.profile.chatback then self:Print(L["disabled"]) end
		end
	end

    -- are we in an Instance/Battleground ?
    if T:IsBattleground(GetRealZoneText()) then
        if self.db.profile.chatback then self:Print(L["battleground"]) end
        self.db.profile.inInstance = true
    elseif T:IsInstance(GetRealZoneText())
	or icecrownInstances[GetRealZoneText()]
	then
        if self.db.profile.chatback then self:Print(L["instance"]) end
        self.db.profile.inInstance = true
    end
    
    self:LevelDebug(3,"inInstance#3: []%s, %s, %s, %s]", self.db.profile.inInstance, T:IsBattleground(GetRealZoneText()),  T:IsInstance(GetRealZoneText()), icecrownInstances[GetRealZoneText()])

    --> Post Event Handler <--
    -- if entering a new area or just been loaded or come out of standby
    if checkEvent[event] or sAFI then
        -- Mute chat in Instances/Battlegrounds
        if self.db.profile.inInstance then
            for channel, off in pairs(self.db.profile.iChannels) do
                if off then
					ChatFrame_RemoveChannel(ChatFrame1, channel)
					self:LevelDebug(2, "Removed CF1 Channel: [%s]", channel)
				end
            end
        else
            for channel, on in pairs(self.db.profile.cf1Channels) do
                if on then
					ChatFrame_AddChannel(ChatFrame1, channel)
					self:LevelDebug(2, "Added CF1 Channel: [%s]", channel)
				end
            end
        end
    end
    
end

function devnull:ChatFrame_OnEvent(this, event, ...)
	self:LevelDebug(1, "ChatFrame_OnEvent:[%s]", event or "???")
	self:LevelDebug(3, "CF_OE:[%s]", ...)

	if self.db.profile[event] then
		-- check to see if the emote is directed at the player
		if event == "CHAT_MSG_TEXT_EMOTE"
		and (string.find(..., UnitName("player"))
		or string.find(..., L["You"]))
		then -- nop
		else
			return
		end
	end
	
	if event == "CHAT_MSG_SYSTEM" then
		if self.db.profile.noDuel
		and string.find(..., L["in a duel"])
		then
			return
		end
		if self.db.profile.noDrunk
		and (string.find(..., L["tipsy"]) or string.find(..., L["drunk"])
		or string.find(..., L["smashed"]) or string.find(..., L["sober"]))
		then
			return
		end
	end

	if event == "CHAT_MSG_BG_SYSTEM_NEUTRAL" then
		if self.db.profile.noDiscovery
		and string.find(..., L["DISCOVERY"])
		then
			return
		end
	end

	self.hooks.ChatFrame_OnEvent(this, event, ...)
	
end

-- Printing Functions
local function makeString(t)

	if type(t) == "table" then
		if type(rawget(t, 0)) == "userdata"
		and type(t.GetObjectType) == "function"
		then
			return ("<%s:%s>"):format(t:GetObjectType(), t:GetName() or "(Anon)")
		end
	end

	return tostring(t)

end
local function makeText(a1, ...)

	local tmpTab = {}
	local output = ""

	if a1:find("%%") and select('#', ...) >= 1 then
		for i = 1, select('#', ...) do
			tmpTab[i] = makeString(select(i, ...))
		end
		output = output .. " " .. a1:format(unpack(tmpTab))
	else
		tmpTab[1] = output
		tmpTab[2] = a1
		for i = 1, select('#', ...) do
			tmpTab[i+2] = makeString(select(i, ...))
		end
		output = table.concat(tmpTab, " ")
	end

	return output

end
local function print(text, frame, r, g, b)

	(frame or DEFAULT_CHAT_FRAME):AddMessage(text, r, g, b, 1, 5)

end

--[===[@debug@
function devnull:Debug(a1, ...)

	local output = ("|cff7fff7f(DBG) %s:[%s.%3d]|r"):format("devnull", date("%H:%M:%S"), (GetTime() % 1) * 1000)

	print(output.." "..makeText(a1, ...), self.debugFrame)

end

function devnull:LevelDebug(lvl, a1, ...) if self.debugLevel >= lvl then self:Debug(a1, ...) end end
--@end-debug@]===]
--@non-debug@
function devnull:Debug() end
function devnull:LevelDebug() end
--@end-non-debug@
